//
//  XSPEC12  November 2003
//
//

#include <XSUser/Global/Global.h>
#include <XSUser/Global/XSGlobal.h>
#include <XSUtil/Utils/XSutility.h>
#include <XSUtil/FunctionUtils/FunctionUtility.h>
#include <XSContainer.h>
#include <XSstreams.h>
#include <XSFit/Fit/Fit.h>
#include <XSModel/GlobalContainer/ModelContainer.h>
#include <XSModel/Model/Model.h>


int
XSGlobal::xsAbund(ClientData cdata,Tcl_Interp* tclInterp,int objc, Tcl_Obj* CONST  objv[])
{
   using namespace XSContainer;
   string cmd = string(static_cast<char*>(cdata));

   try
   {

      if (objc == 1)
      {
         // report current setting
         tcout << FunctionUtility::ABUND() << ": "
               << FunctionUtility::abundDoc(FunctionUtility::ABUND()) 
               << std::endl;
         return globalData->autoSave(TCL_OK);
      }
      else 
      {
         string arg = XSutility::lowerCase(string(Tcl_GetString(objv[1]))); 
         if ( arg[0] == '?')
         {
                 // query options/syntax
                 printDocs(cmd.c_str(),"?");
                 return globalData->autoSave(TCL_OK);       
         } 
         else if (arg == "file")
         {
            if (objc < 3)
            {
               tcerr << "File name must be specified as 3rd argument." <<std::endl;
               return globalData->autoSave(TCL_ERROR);
            }   
            // read the file, and set the abundance
            // vector if successful.
            string file(Tcl_GetString(objv[2]));
            FunctionUtility::readNewAbundances(file);
            // success.
         }  
         else
         {
            if (FunctionUtility::checkAbund(arg))
            {
               FunctionUtility::ABUND(arg);
            }
            else
            {
               // no filename specified so rethrow
               // exception with a message.
               string diag(" solar abundance table ");
               diag += arg;
               diag += ", table not changed.";

               throw FunctionUtility::NoInitializer(diag);
            }
         }
         // Unfortunately we have no way of knowing at the moment if
         // a model contains a component that is affected by the abund
         // change.  Therefore we'll just have to recalculate all the
         // models.
         ModelMap::const_iterator itMod = models->modelSet().begin();
         ModelMap::const_iterator itEnd = models->modelSet().end();
         while (itMod != itEnd)
         {
            Model* mod = itMod->second;
            mod->setComputeFlag(true);
            mod->calculate();
            ++itMod;
         }
         fit->Update();   
      }
   }
   catch ( YellowAlert&)
   {
           // catch instances of NoInitializer or InvalidAbundanceTable
           // and return an error.
           return globalData->autoSave(TCL_ERROR);
   }

   return globalData->autoSave(TCL_OK);
}
